@@ -10,6 +10,8 @@ import os
 import yaml
 import shutil
 from pylabel.shared import _ReindexCatIds
+from tqdm import tqdm
+import csv
 
 from pathlib import PurePath, Path
 
@@ -127,8 +129,12 @@ class Export:
                     segmented_text = ""
 
                 # If the image has no annotations, skip this part of the output
-                if not pd.isnull(df_smaller.loc[index]["cat_id"]):
+                no_annotations = (
+                    pd.isnull(df_smaller.loc[index]["cat_id"])
+                    or df_smaller.loc[index]["cat_id"] == ""
+                )
 
+                if not no_annotations:
                     object_text_start = "<object>"
 
                     name_text = (
@@ -375,7 +381,7 @@ class Export:
                 return output_file_path
 
         # Loop through all images in the dataframe and call voc_xml_file_creation for each one
-        for file_title in list(set(self.dataset.df.img_filename)):
+        for file_title in tqdm(list(set(self.dataset.df.img_filename)), desc='Exporting files'):
 
             file_name = Path(file_title)
             file_name = str(file_name.with_suffix(".xml"))
@@ -401,6 +407,7 @@ class Export:
         copy_images=False,
         use_splits=False,
         cat_id_index=None,
+        segmentation=False,
     ):
         """Writes annotation files to disk in YOLOv5 format and returns the paths to files.
 
@@ -429,6 +436,9 @@ class Export:
                 then increment the cat_ids to index + number of categories continuously.
                 It's useful if the cat_ids are not continuous in the original dataset.
                 Yolo requires the set of annotations to start at 0 when training a model.
+            segmentation (boolean):
+                If true, then segmentation annotations will be exported instead of bounding box annotations.
+                If there are no segmentation annotations, then no annotations will be empty.
 
         Returns:
             A list with 1 or more paths (strings) to annotations files. If a YAML file is created
@@ -481,6 +491,10 @@ class Export:
 
         pd.to_numeric(yolo_dataset["cat_id"])
 
+        if cat_id_index != None:
+            assert isinstance(cat_id_index, int), "cat_id_index must be an int."
+            _ReindexCatIds(yolo_dataset, cat_id_index)
+
         # Convert empty bbox coordinates to nan to avoid math errors
         # If an image has no annotations then an empty label file will be created
         yolo_dataset.ann_bbox_xmin = yolo_dataset.ann_bbox_xmin.replace(
@@ -496,31 +510,21 @@ class Export:
             r"^\s*$", np.nan, regex=True
         )
 
-        if cat_id_index != None:
-            assert isinstance(cat_id_index, int), "cat_id_index must be an int."
-            _ReindexCatIds(yolo_dataset, cat_id_index)
+        # If segmentation = False then export bounding boxes
+        if segmentation == False:
 
-        yolo_dataset["center_x_scaled"] = (
-            yolo_dataset["ann_bbox_xmin"] + (yolo_dataset["ann_bbox_width"] * 0.5)
-        ) / yolo_dataset["img_width"]
-        yolo_dataset["center_y_scaled"] = (
-            yolo_dataset["ann_bbox_ymin"] + (yolo_dataset["ann_bbox_height"] * 0.5)
-        ) / yolo_dataset["img_height"]
-        yolo_dataset["width_scaled"] = (
-            yolo_dataset["ann_bbox_width"] / yolo_dataset["img_width"]
-        )
-        yolo_dataset["height_scaled"] = (
-            yolo_dataset["ann_bbox_height"] / yolo_dataset["img_height"]
-        )
-        yolo_dataset[
-            [
-                "cat_id",
-                "center_x_scaled",
-                "center_y_scaled",
-                "width_scaled",
-                "height_scaled",
-            ]
-        ]
+            yolo_dataset["center_x_scaled"] = (
+                yolo_dataset["ann_bbox_xmin"] + (yolo_dataset["ann_bbox_width"] * 0.5)
+            ) / yolo_dataset["img_width"]
+            yolo_dataset["center_y_scaled"] = (
+                yolo_dataset["ann_bbox_ymin"] + (yolo_dataset["ann_bbox_height"] * 0.5)
+            ) / yolo_dataset["img_height"]
+            yolo_dataset["width_scaled"] = (
+                yolo_dataset["ann_bbox_width"] / yolo_dataset["img_width"]
+            )
+            yolo_dataset["height_scaled"] = (
+                yolo_dataset["ann_bbox_height"] / yolo_dataset["img_height"]
+            )
 
         # Create folders to store annotations
         if output_path == None:
@@ -535,11 +539,11 @@ class Export:
         unique_images = yolo_dataset["img_filename"].unique()
         output_file_paths = []
 
-        for img_filename in unique_images:
+        for img_filename in tqdm(unique_images, desc='Exporting files'):
             df_single_img_annots = yolo_dataset.loc[
                 yolo_dataset.img_filename == img_filename
             ]
-            
+
             basename, _ = os.path.splitext(img_filename)
             annot_txt_file = basename + ".txt"
             # Use the value of the split collumn to create a directory
@@ -554,20 +558,57 @@ class Export:
                 split_dir,
             ).mkdir(parents=True, exist_ok=True)
 
-            df_single_img_annots.to_csv(
-                destination,
-                index=False,
-                header=False,
-                sep=" ",
-                float_format="%.4f",
-                columns=[
-                    "cat_id",
-                    "center_x_scaled",
-                    "center_y_scaled",
-                    "width_scaled",
-                    "height_scaled",
-                ],
-            )
+            # If segmentation = false then output bounding boxes
+            if segmentation == False:
+
+                df_single_img_annots.to_csv(
+                    destination,
+                    index=False,
+                    header=False,
+                    sep=" ",
+                    float_format="%.4f",
+                    columns=[
+                        "cat_id",
+                        "center_x_scaled",
+                        "center_y_scaled",
+                        "width_scaled",
+                        "height_scaled",
+                    ],
+                )
+
+            # If segmentation = true then output the segmentation mask
+            else:
+                # Create one file for image
+                with open(destination, "w") as file:
+                    # Create one row per row in the data frame
+                    for i in range(0, df_single_img_annots.shape[0]):
+                        row = df_single_img_annots.iloc[i].cat_id
+                        segmentation_array = df_single_img_annots.iloc[
+                            i
+                        ].ann_segmentation[0]
+
+                        # Iterate through every value of the segmentation array
+                        # To normalize the coordinates from 0-1
+                        for index, l in enumerate(segmentation_array):
+                            # The first number in the array is the x value so divide by the width
+                            if index % 2 == 0:
+                                row += " " + (
+                                    str(
+                                        segmentation_array[index]
+                                        / df_single_img_annots.iloc[i].img_width
+                                    )
+                                )
+                            else:
+                                # The first number in the array is the x value so divide by the height
+                                row += " " + (
+                                    str(
+                                        segmentation_array[index]
+                                        / df_single_img_annots.iloc[i].img_height
+                                    )
+                                )
+
+                        file.write(row + "\n")
+
             output_file_paths.append(destination)
 
             if copy_images:
@@ -667,7 +708,7 @@ class Export:
         list_c = []
         json_list = []
 
-        for i in range(0, df.shape[0]):
+        for i in tqdm(range(0, df.shape[0]), desc="Exporting files"):
             images = [
                 {
                     "id": df["img_id"][i],
